######################################
#######   Real data analysis   #######
######################################

rm(list = ls())

source("MDS_QP.R")
source("Model_Selection_Criteria.R")


# data source : https://www.investing.com/currencies/usd-isk-historical-data

# data import
isk = read.csv("data/usdisk.csv")
str(isk)

x = isk$date[41:184] # 2004.01 ~ 2015.12
y = isk$price[41:184]

# null check
sum(is.na(x))
sum(is.na(y))

# plot
plot(y, col = 'black', pch = 16, bty = 'n', xlab = "time", ylab = "USD/ISK", xlim = c(0, 160), ylim = c(40, 160), main = "USD/ISK exchange rate")
abline(v = 57, col = 'gray', lty = 2) # 2008.09
# text(58, 40, "2008-09")


#-------------------------------------------------------------------------------#
# Modeling

x = seq(0, 1, length = length(y))

## K, degree
## K = 17, degree = 1
qpsim1 = qp_sim(x, y, K = 17, knots = NULL, degree = 1,
                n_tau = 10, n_t = 50, t_max = NULL, t_max_min_ratio = 1e-4,
                epsilon = 1e-10, verb = T)
plot(y, col = 'gray', bty = 'n')
abline(v = 57, col = 'gray', lty = 2) # 2008.09
points(1 : length(y), qpsim1$best_bic$y_hat, col = 'red', pch = 16)
title("degree = 1, K = 17")
# qpsim1$best_bic$tune

## K = 17, degree = 2
qpsim2 = qp_sim(x, y, K = 17, knots = NULL, degree = 2,
                 n_tau = 10, n_t = 50, t_max = NULL, t_max_min_ratio = 1e-4,
                 epsilon = 1e-10, verb = T)
plot(y, col = 'gray', bty = 'n')
abline(v = 57, col = 'gray', lty = 2) # 2008.09
points(1 : length(y), qpsim2$best_bic$y_hat, col = 'red', pch = 16)
title("degree = 2, K = 17")
# qpsim2$best_bic$tune



## K = 17, degree = 3
qpsim3 = qp_sim(x, y, K = 17, knots = NULL, degree = 3,
                n_tau = 10, n_t = 50, t_max = NULL, t_max_min_ratio = 1e-4,
                epsilon = 1e-10, verb = T)
plot(y, col = 'gray', bty = 'n')
abline(v = 57, col = 'gray', lty = 2) # 2008.09
points(1 : length(y), qpsim3$best_bic$y_hat, col = 'red', pch = 16)
title("degree = 3, K = 17")
# qpsim3$best_bic$tune

#-------------------------------------------------------------------------------#

