
###########################
### Plus Function
###########################
Plus = function(a)
{
  x_plus = a
  x_plus[a < 0] = 0
  return(x_plus)
}


###########################
### Psplines Function
###########################
# K : the number of interior knots
# knots : interior knots

Psplines = function(x, K = NULL, knots = NULL, degree)
{
  # sample size
  N = length(x)
  if(is.null(K))
    K = length(knots)
  else if(is.null(knots))
  {
    # knots = seq(min(x), max(x), length.out = K + 2)
    knots = round(seq(min(x), max(x), length.out = K + 2), 2)
    knots = knots[-c(1, length(knots))]
  }else
    stop("Both K and knots are NULL!")
  
  X = matrix(0, N, K) # basis box
  
  # degree == 0
  if (degree == 0)
  {
    for(k in 1 : K)
    {
      X[, k] = Plus(x - knots[k])
      X[, k][X[, k] > 0] = 1
    }
    X = cbind(1, X)
    dimension = ncol(X)
    
    # basis colnames
    basis_name = c("intercept")
    for(k in 2 : dimension)
      basis_name = c(basis_name, paste0("(x-t_", k - 1, ")_+"))
    colnames(X) = basis_name
    
    #summary
    result = list()
    result$basis = X #matrix
    result$knots = knots
    result$dimension = ncol(X)
    result$degree = degree
    return(result)
  }
  
  # degree > 0
  for(k in 1 : K)
    X[, k] = Plus(x - knots[k])^degree # polynomial
  for(d in degree : 1)
    X = cbind(x^degree, X)             # psplines
  # intercept term
  X = cbind(1, X)
  # dimension
  dimension = ncol(X)
  # basis colnames
  basis_name = c("intercept")
  for(j in 2 : (1 + degree))
    basis_name = c(basis_name, paste0("x^", (j - 1)))
  for(k in 1 : length(knots))
    basis_name = c(basis_name, paste0("(x-t_", k, ")_+"))
  colnames(X) = basis_name
  
  # summary
  result = list()
  result$basis = X
  result$knots = knots
  result$dimension = dimension
  result$degree = degree
  return(result)
}


##################################
### soft-thresholding operator ###
##################################
soft_thresholding = function(z, lambda)
{
  if(z > lambda)
    return(z - lambda)
  if(z < -lambda)
    return(z + lambda)
  return(0)
}
